// -*- mode: js2; indent-tabs-mode: nil; js2-basic-offset: 4 -*-
/*

    Copyright (C) 2012-2017  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Lang = imports.lang;

const GLib = imports.gi.GLib;

const Main = imports.ui.main;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;

const Gettext = imports.gettext;
const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

function _getAccessibility() {
    if (Convenience.checkGSV(3, 4)) {
        try { return Main.panel._statusArea.a11y; } catch(e) { return null; }
    } else {
        try { return Main.panel.statusArea.a11y; } catch(e) { return null; }
    }
}

const GNOME_ACCESSIBILITY = _getAccessibility();

const KEY_SHOW_ACCESSIBILITY = 'show-accessibility';

const PANELMENU_BUTTON_DEFAULT = 0;
const PANELMENU_BUTTON_SHOW = 1;
const PANELMENU_BUTTON_NONE = 2;

let _prototypeQueueSyncMenuVisibility;
let _prototypeSyncMenuVisibility;

function _overrideFunction(enabled) {
    if (enabled) {
        _prototypeSyncMenuVisibility = imports.ui.status.accessibility.ATIndicator.prototype._syncMenuVisibility;
        GNOME_ACCESSIBILITY._syncMenuVisibility = function() {
            let pmShowAccessibility = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA).get_enum(KEY_SHOW_ACCESSIBILITY);
            if (pmShowAccessibility == PANELMENU_BUTTON_DEFAULT) {
                return _prototypeSyncMenuVisibility.call(this);
            } else {
                this.actor.visible = pmShowAccessibility == PANELMENU_BUTTON_SHOW;
                return GLib.SOURCE_REMOVE || false;
            }
        };
        _prototypeQueueSyncMenuVisibility = imports.ui.status.accessibility.ATIndicator.prototype._queueSyncMenuVisibility;
        GNOME_ACCESSIBILITY._queueSyncMenuVisibility = function() {
            if (this._syncMenuVisibilityIdle) return;
            /**
             *  Typing error in accessibility.js:
             *  ::  this._syncMenuVisbilityIdle instead of this._syncMenuVisibilityIdle
             */
            this._syncMenuVisibilityIdle = Mainloop.idle_add(Lang.bind(this, this._syncMenuVisibility));
            if (Convenience.checkGSV(3, 8) || Convenience.checkGSV(3, 10) || Convenience.checkGSV(3, 12)) {
            } else {
                GLib.Source.set_name_by_id(this._syncMenuVisibilityIdle, '[gnome-shell] this._syncMenuVisibility');
            }
        };
    } else {
        GNOME_ACCESSIBILITY._queueSyncMenuVisibility = _prototypeQueueSyncMenuVisibility;
        GNOME_ACCESSIBILITY._syncMenuVisibility = _prototypeSyncMenuVisibility;
    }
}

const ATIndicator = new Lang.Class({
    Name: 'ATIndicator',

    _init: function() {
        if (!Convenience.checkGSV(3, 4) && !Convenience.checkGSV(3, 6)) _overrideFunction(true);
        this._itemScreenReader = this._checkScreenReader();
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = this._settings.connect('changed::'+KEY_SHOW_ACCESSIBILITY, Lang.bind(this, this._onShowAccessibility));
        this._onShowAccessibility();
    },

    _checkScreenReader: function() {
        if (!Convenience.checkGSV(3, 4)) return null;
        /**
         *  Missing ScreenReader item on GNOME-Shell 3.4.x (deactivated in source)
         */
        let menuItems = GNOME_ACCESSIBILITY.menu._getMenuItems();
        for each(let item in menuItems) {
            if (item.label && item.label.text == _("Screen Reader")) return null;
        }
        let item = GNOME_ACCESSIBILITY._buildItem(_("Screen Reader"), 'org.gnome.desktop.a11y.applications', 'screen-reader-enabled');
        GNOME_ACCESSIBILITY.menu.addMenuItem(item, 3);
        return item;
    },

    _onShowAccessibility: function() {
        this._updateVisibility(this._settings.get_enum(KEY_SHOW_ACCESSIBILITY));
    },

    _updateVisibility: function(showAccessibility) {
        if (Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6)) {
            GNOME_ACCESSIBILITY.actor.visible = showAccessibility != PANELMENU_BUTTON_NONE;
        } else {
            GNOME_ACCESSIBILITY._syncMenuVisibility();
        }
    },

    destroy: function() {
        if (!Convenience.checkGSV(3, 4) && !Convenience.checkGSV(3, 6)) _overrideFunction(false);
        this._settings.disconnect(this._connectSettings);
        if (this._itemScreenReader) this._itemScreenReader.destroy();
        this._updateVisibility(PANELMENU_BUTTON_DEFAULT);
    }
});

function getExtension() {
    return GNOME_ACCESSIBILITY ? new ATIndicator() : null;
}
