// -*- mode: js2; indent-tabs-mode: nil; js2-basic-offset: 4 -*-
/*

    Copyright (C) 2012-2017  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Lang = imports.lang;

const Gtk = imports.gi.Gtk;
const GObject = imports.gi.GObject;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;

const Gettext = imports.gettext;
const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

const KEY_DISPLAY_TEMPERATURE = 'display-temperature';
const KEY_REFRESH_INTERVAL = 'refresh-interval';
const KEY_SHOW_CPU_CORE_TEMPERATURE = 'show-cpu-core-temperature';
const KEY_SHOW_DISK_TEMPERATURE = 'show-disk-temperature';
const KEY_SHOW_FAHRENHEIT = 'show-fahrenheit';
const KEY_SHOW_GRAPHICS_TEMPERATURE = 'show-graphics-temperature';

const SETTINGS_GLOBAL = [
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'switcher',
        key:     KEY_SHOW_FAHRENHEIT,
        label:   _("View Fahrenheit"),
        tooltip: _("If true, display the temperature in Fahrenheit instead of Celsius.")
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_REFRESH_INTERVAL,
        label:   _("Refresh interval"),
        tooltip: _("Select the desired refresh interval in seconds."),
        values:  { '10':  '10',
                   '15':  '15',
                   '30':  '30',
                   '60':  '60',
                   '120': '120',
                   '300': '300',
                   '600': '600' }
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_DISPLAY_TEMPERATURE,
        label:   _("Display Temperature"),
        tooltip: _("Select the temperature that displayed in the GNOME Shell panel."),
        values:  { 'cpu':       _("CPU"),
                   'cpu-cores': _("CPU-Core"),
                   'vga':       _("Graphics Card"),
                   'cpu-vga':   _("CPU / Graphics Card") }
    }
];

const SETTINGS_DISPLAY = [
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_SHOW_CPU_CORE_TEMPERATURE,
        label:   _("Show CPU Core"),
        tooltip: _("Select the display mode for CPU Core temperature."),
        values:  { 'none':     _("none"),
                   'current':  _("current"),
                   'critical': _("current / critical"),
                   'maximum':  _("current / maximum"),
                   'all':      _("current / maximum / critical") }
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'switcher',
        key:     KEY_SHOW_DISK_TEMPERATURE,
        label:   _("Show Disk"),
        tooltip: _("If true, display temperature for Disk.")
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_SHOW_GRAPHICS_TEMPERATURE,
        label:   _("Show Graphics Card"),
        tooltip: _("Select the display mode for Graphics Card temperature."),
        values:  { 'none':     _("none"),
                   'current':  _("current"),
                   'critical': _("current / critical"),
                   'maximum':  _("current / maximum"),
                   'all':      _("current / maximum / critical") }
    }
];

const TemperaturePrefsWidget = new GObject.Class({
    Name: 'Temperature.Prefs.Widget',
    GTypeName: 'TemperaturePrefsWidget',
    Extends: Gtk.Notebook,

    _init: function(params) {
        this.parent(params);
        this._createPrefsWidget();
    },

    _createPrefsWidget: function() {
        let box;
        box = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL, margin: 15, spacing: 15 });
        for (i in SETTINGS_GLOBAL) {
            box.add(this._createWidget(SETTINGS_GLOBAL[i]));
        }
        this.append_page(box, new Gtk.Label({ label: _("General") }));
        box = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL, margin: 15, spacing: 15 });
        for (i in SETTINGS_DISPLAY) {
            box.add(this._createWidget(SETTINGS_DISPLAY[i]));
        }
        this.append_page(box, new Gtk.Label({ label: _("Display") }));
    },

    _createWidget: function(args) {
        let box;
        let settings = Convenience.getSettings(args['schema']);
        let widget;
        box = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL });
        box.pack_start(new Gtk.Label({ label: _(args['label']), xalign: 0 }), true, true, 0);
        switch (args['type']) {
            case 'combo':
                widget = new Gtk.ComboBoxText();
                for (let i in args['values']) {
                    widget.append(i, _(args['values'][i]));
                }
                widget.set_active_id(settings.get_string(args['key']));
                widget.width_request = 200;
                widget.connect('changed', function() {
                    settings.set_string(args['key'], widget.get_active_id());
                });
                settings.connect('changed::'+args['key'], function() {
                    widget.set_active_id(settings.get_string(args['key']));
                });
                break;
            case 'switcher':
                widget = new Gtk.Switch({ active: settings.get_boolean(args['key']) });
                widget.connect('notify::active', function() {
                    settings.set_boolean(args['key'], widget.get_active());
                });
                settings.connect('changed::'+args['key'], function() {
                    widget.set_active(settings.get_boolean(args['key']));
                });
                break;
        }
        if (args['tooltip']) widget.set_tooltip_text(_(args['tooltip']));
        box.pack_end(widget, false, false, 0);
        return box;
    }
});

function init() {
    Convenience.initTranslations();
}

function buildPrefsWidget() {
    let widget = new TemperaturePrefsWidget({
        scrollable: true,
        margin: 15,
        width_request: 720,
        height_request: 480
    });
    widget.show_all();
    return widget;
}
